import { Component, OnInit, Input } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { BackendService, ScrumUserstory, Priority } from '../services/backend.service';
import {
	ScrumTask,
	ScrumStatus,
	ScrumCategory,
	ScrumUser,
	ScrumProject,
} from '../services/backend.service';

@Component({
	selector: 'app-userstory-form',
	templateUrl: './userstory-form.component.html',
	styleUrls: ['./userstory-form.component.css']
})
export class UserstoryFormComponent implements OnInit {
	@Input() public userstory: ScrumUserstory;
	public allStatus: any[] = [];
	public status: ScrumStatus = { title: "", description: "" };
	public allUser: any[] = [];
	public user: ScrumUser = { name: "" };
	public allCategories: any[] = [];
	public category: ScrumCategory = { title: ""};
	private editing: boolean;


	constructor(private backendService: BackendService, private activeModalService: NgbActiveModal) {
		this.getUserstoryStatus();
		this.getAllUsers();
		this.getUserstoryCategory();
	}

	ngOnInit(): void {
		if (this.userstory === null || this.userstory === undefined) {
			this.userstory = { title: '' };
			this.editing = false;
		} else {
			this.editing = true;
		}
		document.getElementById('titleField').focus();
	}

	onSubmit() {
		if (this.editing) {
			this.backendService.putUserstory(this.userstory).subscribe((response) => {
				if (response.status > 399) {
					alert('Fehler');
				}
			});
		} else {
			this.backendService.postUserstory(this.userstory).subscribe((response) => {
				if (response.status > 399) {
					alert('Fehler');
				}
			});
		}
		this.activeModalService.close(this.userstory);
	}
	onClose() {
		this.activeModalService.dismiss(this.userstory);
	}

	// Methods for adding, choosing and deleting random status tags.

	getUserstoryStatus() {
		this.backendService.getAllStatus().subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			} else {
				this.allStatus.push(...response.body);
			}
		});
	}

	createUserstoryStatus(status: ScrumStatus) {
		this.backendService.postStatus(status).subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			}
			else {
				this.allStatus.push(response.body);
			}
		});
	}

	deleteStatus(id: number) {
		var status = this.allStatus.find((x) => x.id === id);
		this.backendService.deleteStatus(status).subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			}
			else {
				const index = this.allStatus.indexOf(status);
				if (index !== -1) {
					this.allStatus.splice(index, 1);
				}
			}
			this.userstory.statusid = null;
		});
	}

	getAllPriorities(): Priority[] {
		return Object.values(Priority);
	}

	getStatusTitleById(id: number): string {
		if (!id) {
			return null;
		}
		var status = this.allStatus.find((x) => x.id === id);
		if (!status) {
			return null;
		}
		return status.title;
	}

	// Methods for choosing creator of a userstory.

	getAllUsers() {
		this.backendService.getUsers().subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			} else {
				this.allUser.push(...response.body);
			}
		});
	}

	getAuthorById(id: number): string {
		if (!id) {
			return null;
		}
		var user = this.allUser.find((x) => x.id === id);
		if (!user) {
			return null;
		}
		return user.name;
	}

	// Methods for creating, choosing and deleting categories for a userstory.

	getUserstoryCategory() {
		this.backendService.getCategories().subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			} else {
				this.allCategories.push(...response.body);
			}
		});
	}

	createUserstoryCategory(category: ScrumCategory) {
		this.backendService.postCategory(category).subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			}
			else {
				this.allCategories.push(response.body);
			}
		});
	}

	deleteCategory(id: number) {
		var category = this.allCategories.find((x) => x.id === id);
		this.backendService.deleteCategory(category).subscribe((response) => {
			if (response.status > 399) {
				alert('Fehler');
			}
			else {
				const index = this.allCategories.indexOf(category);
				if (index !== -1) {
					this.allCategories.splice(index, 1);
				}
			}
			this.userstory.categoryid = null;
		});
	}

	getCategoryById(id: number): string {
		if (!id) {
			return null;
		}
		var category = this.allCategories.find((x) => x.id === id);
		if (!category) {
			return null;
		}
		return category.title;
	}

}
